%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Code for model described in Lyttle et al (2013).
% Note! - The network size here has been scaled down considerably so that
% the code will run on a typical desktop PC. Therefore the results will be
% quite different (but qualitatively similar) to those presented in
% the paper. To reproduce those results, change the parameters marked with
% a ** to what was specified in the paper, and run this code on a high 
% performance computing system with a lot of memory. 
% 
% Contact dlyttle@math.arizona.edu with questions or comments.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


clear all
tic

%%%% file in which to save the results
savefile = 'example_data.mat';

%%%% Parameters
nPlaceMod = 30; %number of place cell groups **rescaled from manuscript**
nGridMod = 11;  %number of grid cell modules
N =200; %number of grid cells per module **rescaled from manuscript**
ip = 0.5; % 1-alpha controls grid cell input overlap
cp = 1; % controls e%-max overlap **rescaled from manuscript**
maxintensity = 2; %maximum firing rate for non-spatial cells
propnonmin = 0.2; %baseline proportion of nonspatial excitation for dorsal cells
propnon = 0.85; % maximum proportion of nonspatial input at ventral pole
intot = 50; % number of grid cell inputs **rescaled from manuscript**
comptot = 200; %number of place cells per module **rescaled from manuscript**
sp = linspace(30,100,nGridMod); %vector of spatial scales for the grid modules
cut=.9; %E%-max cutoff
minSize = 50; %minimum size for an active region to count as a "place field"

numit=10; %number of independent networks to run

%store and save the parameter values
params = [nPlaceMod nGridMod N ip cp maxintensity propnonmin propnon intot comptot sp cut minSize];

%grid node variability parameters
mu=1; %mean
sigd = 0.5; %standard deviation
xlo = 0;  %min
xhi = 2; %max
plo=normcdf((xlo-mu)/sigd);
phi=normcdf((xhi-mu)/sigd);

%scale the number of nonspatial inputs as a function of the mean intensity
minnon = (intot*propnonmin*0.24)/((maxintensity/2) - (maxintensity/2)*propnonmin);
maxnon = (intot*propnon*0.24)/((maxintensity/2) - (maxintensity/2)*propnon);
numnon = linspace(minnon,maxnon,nPlaceMod);

%form the input and E% overlap distributions
scaling1 = zeros(1,nPlaceMod);
scaling2 = zeros(1,nPlaceMod);
indistraw = zeros(nGridMod,nGridMod);
indist = zeros(nPlaceMod,nGridMod);
compdist = zeros(nPlaceMod,nPlaceMod);
for x = 0:(nGridMod-1)
    scaling1 = round(intot/(geocdf((nGridMod-1)-x,ip)+geocdf(x,ip)-geocdf(0,ip)));
    if x<(nGridMod-1)
        indistraw(x+1,:) = round(scaling1*horzcat(fliplr(geopdf(0:x,ip)),geopdf(1:((nGridMod-1)-x),ip)));
    else
        indistraw(x+1,:) = round(scaling1*fliplr(geopdf(0:x,ip)));
    end
end
for base = 0:(nGridMod-2)
   for add = 1:(nPlaceMod/(nGridMod-1)) 
    fracnew = (add-1)/(nPlaceMod/(nGridMod-1)); 
    indist(base*(nPlaceMod/(nGridMod-1))+add,:) = (1-fracnew)*indistraw(base+1,:) + fracnew*indistraw(base+2,:);
   end
end
for y = 0:(nPlaceMod-1)
    scaling2 = round(comptot/(geocdf((nPlaceMod-1)-y,cp)+geocdf((nPlaceMod-1),cp)-geocdf(0,cp)));
    if y<(nPlaceMod-1)
        compdist(y+1,:) = round(scaling2*horzcat(fliplr(geopdf(0:y,cp)),geopdf(1:((nPlaceMod-1)-y),cp)));
    else
        compdist(y+1,:) = round(scaling2*fliplr(geopdf(0:y,cp)));
    end
end

%iterate and collect statistics for several independent network realizations
for it = 1:numit
fprintf('Independent Network %d\n',it)      
    %% Build 8 sets of grid cells each with a (noisy) shared orientation and increasing spatial scale. Also build nonspatial inputs
%generate random collection of phases
p = randi(sp(nGridMod),2,N,nGridMod);
%generate random collecton of orientations
oseed = randi(60,1,nGridMod);%randperm(8);
for gg=1:nGridMod
o(:,gg) = oseed(gg)+ randi(15,1,N);
end

a=0.3;
b=-3/2;
[X,Y]=meshgrid(1:100);
gcells = zeros(100,100,N,nGridMod);
for mm = 1:nGridMod
for n=1:N
    %it
    %n
    %script to generate gridcells
    c1=p(1,n,mm);
    c2=p(2,n,mm);
    s=sp(mm);
    r=o(n,mm);
    g= zeros(100,100);
    input = cos((4*pi/(sqrt(3)*s))*(cosd(-30+r)*(X-c1)+sind(-30+r)*(Y-c2))) + cos((4*pi/(sqrt(3)*s))*(cosd(30+r)*(X-c1)+sind(30+r)*(Y-c2))) + cos((4*pi/(sqrt(3)*s))*(cosd(90+r)*(X-c1)+sind(90+r)*(Y-c2)));
    g = exp(a*(input-b)) - 1;
    
    %%this bit of code introduces variability into the grid cell node
    %%peaks
    g = g./max(max(g));
    g(g<.1) = 0;
    blebs= bwconncomp(g,4);
    % generate uniform [0,1] random deviates
    r=rand(1,blebs.NumObjects);
    % scale to [plo,phi]
    r=plo+(phi-plo)*r;
    % Invert through standard normal
    z=norminv(r);
    % apply shift and scale
    peaks=mu+z*sigd;
    %peaks = rand(1,blebs.NumObjects);
    for nn = 1:blebs.NumObjects
        g(blebs.PixelIdxList{nn}) = g(blebs.PixelIdxList{nn})*peaks(nn);
    end
    %end bit
    gcells(:,:,n,mm) = g;
end
end

%make the pool of nonspatial inputs
noncells = maxintensity*rand(1,nGridMod*N);

%% generate connectivity patterns

clear indices;
clear weights;
warning('off','all')

for qq = 1:nPlaceMod %to place module
    for pp = 1:nGridMod %from grid module
        indices{pp}{qq}(:,:) = randi(N,indist(qq,pp),compdist(qq,qq));
        weights{pp}{qq}=rand(size(indices{pp}{qq}));
    end
end

%% compute the excitation maps for all the cells
%initialize the excitation arrays
ei = cell(1,nPlaceMod);
for q = 1:nPlaceMod
    ei{q} = zeros(100,100,compdist(q,q));
end

%convert grid cells to vector form for easy multiplication
gcellvec = cell(1,nGridMod);
for jj = 1:nGridMod
    %ggroup = gcells(:,:,:,jj);
    gcellvec{jj} = reshape(gcells(:,:,:,jj), [100*100, N]);
end
clear gcells;

%form the excitation maps
for jj = 1:nPlaceMod
    fprintf('--Place cell group: %d\n',jj)     
    for j =1:compdist(jj,jj) 
        %j
        emap = zeros(100*100,nGridMod);
        for mm = 1:nGridMod
            if ~isempty(indices{mm}{jj}(:,j))
                ind1 = indices{mm}{jj}(:,j);
                sel1 = gcellvec{mm}(:,ind1);
                wei1 = weights{mm}{jj}(:,j);
                %wei1 = rand(size(ind1));
                ein =  sel1*wei1;
                emap(:,mm) = ein;
            end
        end     
        em = sum(emap,2);       
        nonweight = rand(1,round(numnon(jj)));
        nonspat = sum(nonweight.*noncells(randi(nGridMod*N,1,round(numnon(jj)))));
        em = em + nonspat;
        ei{jj}(:,:,j) = reshape(em,[100,100]);
    end
end
clear gcellvec; 


%% compute the place fields for all the cells, with competition overlapping across groups

%initialize the max maps
maxmap = zeros(100,100,nPlaceMod);

%form the max maps
for kk = 1:nPlaceMod
    mmap = zeros(100,100,nPlaceMod);
   for k = 1:nPlaceMod 
%      if k == kk
%       mmap(:,:,k) = max(ei{k}(:,:,:),[],3);
      if compdist(kk,k) >0
       %comp = randi(compdist(k,k),1,compdist(kk,k));
       comp = 1:compdist(kk,k);
       mmap(:,:,k) = max(ei{k}(:,:,comp),[],3);
      else
       mmap(:,:,k) = zeros(100,100);
      end
   end
   maxmap(:,:,kk)= max(mmap,[],3);
end

%%% form and collect statistics on the place fields
numbers = cell(1,nPlaceMod);
siz = cell(1,nPlaceMod);
active = cell(1,nPlaceMod);
bits = cell(1,nPlaceMod);
bigf = cell(1,nPlaceMod);

%iterate over place cell groups
for xx= 1:nPlaceMod
    numfields = zeros(1,compdist(xx,xx)); %vector containing number of fields per cell
    fieldsizes = []; %vector of individual field sizes
    covered = []; %vector containing total area covered by the fields of each cell
    largest = zeros(1,compdist(xx,xx)); %vector containing area of the largest field of each cell
    %iterate over cells
    for k = 1:compdist(xx,xx) 
        % form the fields
        f1 = ((ei{xx}(:,:,k)-cut.*maxmap(:,:,xx)).*(ei{xx}(:,:,k)>=cut.*maxmap(:,:,xx)));
        % imagesc(f1); drawnow;     % uncomment this is you want to see the
        % place fields. Will slow things down quite a bit!.
        blobs = bwconncomp(f1,4); 
        nf = 0;
        coverage = 0;
        biggie = 0;
        for jj = 1:blobs.NumObjects
        if length(blobs.PixelIdxList{jj}) >= minSize
            nf = nf+1;
            fieldsizes = [fieldsizes, length(blobs.PixelIdxList{jj})];
            coverage = coverage + length(blobs.PixelIdxList{jj});
            if length(blobs.PixelIdxList{jj}) >= biggie
               biggie = length(blobs.PixelIdxList{jj}); 
            end
        end
        end
        covered = [covered coverage];
        largest(k) = biggie;
        numfields(k) = nf;
    end
    numbers{xx} = numfields;
    siz{xx} = fieldsizes;
    active{xx} = covered;
    bigf{xx} = largest;
end
clear ei;

%% save the results
output{it} = {numbers, siz, active, bigf}; 

end
toc;

%save the place field statistics across all networks
save(savefile, 'output', 'params');


%%%% To analyze and plot the data, use the associated script plotsizedist.m
